
# These two lines are needed to run on EL6
__requires__ = ['SQLAlchemy >= 0.7', 'jinja2 >= 2.4']
import pkg_resources

import default_config
import model

model.create_tables(
    default_config.DB_URL,
    None,
    debug=True)



import redis

ports = [2222, 2223, 2224,2225,2226, 2227, 2228, 2229, 2230]

if __name__ == '__main__':
    r = redis.Redis()
    print "First cleaning the existing ports."
    r.delete("tunirports")
    for p in ports:
        r.lpush('tunirports', p)



# -*- coding: utf-8 -*-
"Tunir module to talk to Vagrant"

# Copyright © 2015  Kushal Das <kushaldas@gmail.com>
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2, or (at your option) any later
# version.  This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.  You
# should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import os
import time
import subprocess


def system(cmd):
    """
    Runs a shell command, and returns the output, err, returncode

    :param cmd: The command to run.
    :return:  Tuple with (output, err, returncode).
    """
    ret = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
    out, err = ret.communicate()
    returncode = ret.returncode
    return out, err, returncode

def parse_ssh_config(text):
    """
    Parses the SSH config and returns a dict
    """
    result = {}
    lines = text.split('\n')
    if lines[0].strip().startswith('Host '):
        for line in lines[1:]:
            line = line.strip()
            words = line.split(' ')
            if len(words) == 2:
                result[words[0]] = words[1]

    return result


class Vagrant(object):
    """
    Returns a Vagrant object.
    """
    def __init__(self, image_url, name='tunir-box', memory=1024):
        self.original_path = os.path.abspath(os.path.curdir)
        self.name = name
        self.image_url = image_url
        self.path = '/var/run/tunir/'
        self.keys = None


        os.chdir(self.path)
        with open('Vagrantfile', 'w') as fobj:
            fobj.write('''Vagrant.configure("2") do |config|
  config.vm.define :tunirserver do |tunirserver|
    tunirserver.vm.box = "{0}"
    tunirserver.vm.provider :libvirt do |domain|
      domain.memory = {1}
      domain.cpus = 2
    end
  end
end'''.format(name, memory))

        print "Wrote Vagrant config file."
        # Now actually register that image

        basename = os.path.basename(image_url)


        print "Adding vagrant box."
        cmd = 'vagrant box add {0} --name {1}'.format(image_url, name)
        out, err, retcode = system(cmd)
        # Now check for error, I will skip this.
        if retcode != 0:
            print("Error while trying to add the box.")
            return
        print out

        print "Up the vagrant"

        # Let us up the vagrant
        cmd = 'vagrant up'
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to do vagrant up the box.")
            return
        print out

        time.sleep(3)
        print "Time to get Vagrant ssh-config"


        # Now let us try to get the ssh-config
        cmd = 'vagrant ssh-config'
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to get ssh config for the box.")
            return
        print out
        self.keys = parse_ssh_config(out)
        os.chdir(self.original_path)

    def destroy(self):
        os.chdir(self.path)
        print "Let us destroy the box."
        cmd = 'vagrant destroy'
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to destroy the instance.")
            return

        cmd = 'vagrant box remove {0}'.format(self.name)
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to remove the box.")
            return
        os.chdir(self.original_path)

def vagrant_and_run(config):
    """
    This starts the vagrant box

    :param config: Our config object
    :return: (Vagrant, config) config object with IP, and key file
    """

    v = Vagrant(config['image'], config['ram'])
    if v.keys: # Means we have the box up, and also the ssh config
        config['host_string'] = v.keys['HostName']
        config['key'] = v.keys['IdentityFile']

    return v, config




if __name__ == '__main__':
    data = '''Host default
  HostName 192.168.121.18
  User vagrant
  Port 22
  UserKnownHostsFile /dev/null
  StrictHostKeyChecking no
  PasswordAuthentication no
  IdentityFile /root/vg/.vagrant/machines/default/libvirt/private_key
  IdentitiesOnly yes
  LogLevel FATAL
'''
    print parse_ssh_config(data)




# -*- coding: utf-8 -*-
"Tunir module to talk to docker"

# Copyright © 2015  Kushal Das <kushaldas@gmail.com>
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2, or (at your option) any later
# version.  This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.  You
# should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import json
import subprocess


def system(command):
    """
    Executes a given command
    :param command: the command to run
    :return: Tuple of the output and error message.
    """
    ret = subprocess.Popen(command, shell=True, stdin=subprocess.PIPE,
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
    out, err = ret.communicate()
    return out, err

class Result(str):
    """
    To hold results from docker command executions.
    """
    @property
    def stdout(self):
        return str(self)

class Docker(object):
    """
    Returns a Docker object. The image must have sshd running.
    """
    def __init__(self, image):
        self.image = image
        self.cid = None
        # Now we will start a new container with the given container name
        out, code = system('docker run -d -p 22 %s' % image)
        if not code:
            self.cid = out.strip('\n')
        else:
            print "Some error in creating the container."
            return
        out, code = system('docker inspect %s' % self.cid)
        data = json.loads(out)
        self.ip  = data[0]['NetworkSettings']['IPAddress']

    def rm(self):
        "Removes the current container."
        system('docker rm --force %s' % self.cid)






if __name__ == '__main__':
    d = Docker('kushaldas/ssh1')
    print d.cid, d.ip
    d.rm()


LOCAL_DOWNLOAD_DIR = "/tmp/"
DOWNLOAD_PROGRESS = True

# Data for cloud-init

META_DATA =  """instance-id: iid-123456
local-hostname: %s
"""
USER_DATA = """#cloud-config
password: %s
chpasswd: { expire: False }
ssh_pwauth: True
""" 
ATOMIC_USER_DATA = """#cloud-config
password: %s
chpasswd: { expire: False }
ssh_pwauth: True
runcmd:
  - [ sh, -c, 'echo -e "ROOT_SIZE=4G\nDATA_SIZE=10G" > /etc/sysconfig/docker-storage-setup']
""" 



import os
import sys
import json
import time
import redis
import signal
import argparse
import tempfile
import shutil
import paramiko
from pprint import pprint
from testvm import build_and_run
from tunirvagrant import vagrant_and_run
from tunirdocker import Docker, Result
from collections import OrderedDict

STR = OrderedDict()


def run(host='127.0.0.1', port=22, user='root',
                  password='passw0rd', command='/bin/true', bufsize=-1, key_filename=''):
    """
    Excecutes a command using paramiko and returns the result.
    :param host: Host to connect
    :param port: The port number
    :param user: The username of the system
    :param password: User password
    :param command: The command to run
    :param key_filename: SSH private key file.
    :return:
    """
    port = int(port)
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    if not key_filename:
        client.connect(hostname=host, port=port,
                   username=user, password=password, banner_timeout=10)
    else:
        print host, port, user, key_filename
        client.connect(hostname=host, port=port,
                   username=user, key_filename=key_filename, banner_timeout=10)
    chan = client.get_transport().open_session()
    chan.settimeout(None)
    chan.set_combine_stderr(True)
    chan.exec_command(command)
    stdout = chan.makefile('r', bufsize)
    stderr = chan.makefile_stderr('r', bufsize)
    stdout_text = stdout.read()
    stderr_text = stderr.read()
    out = Result(stdout_text)
    status = int(chan.recv_exit_status())
    client.close()
    out.return_code = status
    return out


def read_job_configuration(jobname='', config_dir='./'):
    """
    :param jobname: Name of the job
    :param config_dir: Directory for configuration.
    :return: Configuration dict
    """
    data = None
    name = jobname + '.json'
    name = os.path.join(config_dir, name)
    if not os.path.exists(name):
        print "Job configuration is missing."
        return None
    with open(name) as fobj:
        data = json.load(fobj)
    return data

def try_again(func):
    "We will try again for ssh errors."
    def wrapper(*args, **kargs):
        try:
            result = func(*args, **kargs)
        except paramiko.ssh_exception.SSHException:
            print "Getting ssh exception, sleeping for 30 seconds and then trying again."
            time.sleep(30)
            result = func(*args, **kargs)
        return result
    return wrapper

@try_again
def execute(config, command, container=None):
    """
    Executes a given command based on the system.
    :param config: Configuration dictionary.
    :param command: The command to execute
    :return: (Output text, boolean)
    """
    result = ''
    negative = False
    if command.startswith('@@'):
        command = command[3:].strip()
        result = run(config['host_string'], config.get('port', '22'), config['user'],
                         config.get('password', None), command, key_filename=config.get('key', None))
        if result.return_code != 0:  # If the command does not fail, then it is a failure.
            negative = True
    else:
        result = run(config['host_string'], config.get('port', '22'), config['user'],
                        config.get('password', None), command, key_filename=config.get('key', None))
    return result, negative

def update_result(result, command, negative):
    """
    Updates the result based on input.

    :param result: Output from the command
    :param job: Job object from model.
    :param command: Text command.
    :param negative: If it is a negative command, which is supposed to fail.
    :param stateless: If it is a stateless job or not.

    :return: Boolean, False if the job as whole is failed.
    """
    if negative:
        status = True
        if result.return_code == 0:
            status = False
        d = {'command': command, 'result': unicode(result, encoding='utf-8', errors='replace'),
             'ret': result.return_code, 'status': status}
        STR[command] = d

    else:
        status = True
        if result.return_code != 0:
            status = False
        d = {'command': command, 'result': unicode(result, encoding='utf-8', errors='replace'),
             'ret': result.return_code, 'status': status}
        STR[command] = d

    if result.return_code != 0 and not negative:
        # Save the error message and status as fail.
        return False

    return True


def run_job(args, jobpath, job_name='', config=None, container=None, port=None):
    """
    Runs the given command using fabric.

    :param args: Command line arguments.
    :param jobpath: Path to the job file.
    :param job_name: string job name.
    :param config: Configuration of the given job
    :param container: Docker object for a Docker job.
    :param port: The port number to connect in case of a vm.
    :return: Status of the job in boolean
    """
    if not os.path.exists(jobpath):
        print "Missing job file {0}".format(jobpath)
        return False

    # Now read the commands inside the job file
    # and execute them one by one, we need to save
    # the result too.
    commands = []
    status = True

    with open(jobpath) as fobj:
        commands = fobj.readlines()


    try:
        job = None

        print "Starting a stateless job."

        if not 'host_string' in config: # For VM based tests.
            config['host_string'] = '127.0.0.1'

        if config['type'] == 'vm':
            config['port'] = port
        elif config['type'] == 'bare':
            config['host_string'] = config['image']
        elif config['type'] == 'docker':
            # Now we will convert this job as a bare metal :)
            config['type'] = 'bare'
            config['host_string'] = container.ip
            time.sleep(10)
        for command in commands:
            negative = False
            result = ''
            command = command.strip('\n')
            if command.startswith('SLEEP'): # We will have to sleep
                word = command.split(' ')[1]
                print "Sleeping for %s." % word
                time.sleep(int(word))
                continue
            print "Executing command: %s" % command

            result, negative = execute(config, command)
            status = update_result(result, command, negative)
            if not status:
                break

        # If we are here, that means all commands ran successfully.

    finally:
        # Now for stateless jobs
        print "\n\nJob status: %s\n\n" % status

        for key, value in STR.iteritems():
            print "command: %s" % value['command']
            print "status: %s\n" % value['status']
            print value['result']
            print "\n"

        return status


def get_port():
    "Gets the latest port from redis queue."
    r = redis.Redis()
    port = r.rpop('tunirports')
    print "Got port: %s" % port
    return port

def return_port(port):
    """
     Returns the port to the queue.
    :param port: The port number
    :return: None
    """
    r = redis.Redis()
    port = r.lpush('tunirports', port)
    return port

def main(args):
    "Starting point of the code"
    job_name = ''
    vm = None
    port = None
    temp_d = None
    container = None
    atomic = False
    vagrant = None
    return_code = -100
    if args.atomic:
        atomic = True
    if args.job:
        job_name = args.job
    else:
        sys.exit(-2)

    # First let us read the vm configuration.
    config = read_job_configuration(job_name, args.config_dir)
    if not config: # Bad config name
        sys.exit(-1)

    if config['type'] in ['vm',]:
        # Now we need a temporary directory
        temp_d = tempfile.mkdtemp()
        os.system('chmod 0777 %s' % temp_d)
        os.mkdir(os.path.join(temp_d, 'meta'))


    if config['type'] == 'vm':
        # First get us the free port number from redis queue.
        port = get_port()
        if not port:
            print "No port found in the redis queue."
            return
        vm = build_and_run(config['image'], config['ram'],
                           graphics=True, vnc=False, atomic=atomic,
                           port=port, temppath=temp_d)
        job_pid = vm.pid # The pid to kill at the end
        # We should wait for a minute here
        time.sleep(60)
    if config['type'] == 'docker':
        container = Docker(config['image'])
    jobpath = os.path.join(args.config_dir, job_name + '.txt')

    if config['type'] == 'vagrant':
        vagrant, config = vagrant_and_run(config)

    try:
        status = run_job(args, jobpath, job_name, config, container, port)
        if status:
            return_code = 0
    finally:
        # Now let us kill the kvm process
        if vm:
            os.kill(job_pid, signal.SIGKILL)
            if temp_d:
                shutil.rmtree(temp_d)
            return_port(port)
        if container:
            container.rm()
        if vagrant:
            print "Removing the box."
            vagrant.destroy()
        else:
            # FIXME!!!
            # Somehow the terminal is not echoing unless we do the line below.
            os.system('stty sane')
        sys.exit(return_code)


def startpoint():
    parser = argparse.ArgumentParser()
    parser.add_argument("--job", help="The job configuration name to run")
    parser.add_argument("--stateless", help="Do not store the result, just print it in the STDOUT.", action='store_true')
    parser.add_argument("--config-dir", help="Path to the directory where the job config and commands can be found.",
                        default='./')
    parser.add_argument("--atomic", help="We are using an Atomic image.", action='store_true')
    args = parser.parse_args()

    main(args)

if __name__ == '__main__':
    startpoint()



# -*- coding: utf-8 -*-
# Copyright 2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

"""
This is a module for downloading fedora cloud images (and probably any other
qcow2) and then booting them locally with qemu.
"""

import os
import glob
import subprocess
import sys
import urllib2
import shutil

import config


def koji_download(urls, download_path):
    """ Downloads files (qcow2s, specifically) from a list of URLs with an
    optional progress bar. Returns a list of raw image files. """

    # This code was blatantly stolen from fedimg - but it was depreciated,
    # that's the internet version of sitting in front of someone's house with
    # a sign saying "FREE." Thanks oddshocks!

    # Create the proper local upload directory

    print "Local downloads will be stored in {}.".format(
        download_path)

    # When qcow2s are downloaded and converted, they are added here
    raw_files = list()

    for url in urls:
        file_name = url.split('/')[-1]
        local_file_name = os.path.join(download_path, file_name)
        u = urllib2.urlopen(url)
        try:
            with open(local_file_name, 'wb') as f:
                meta = u.info()
                file_size = int(meta.getheaders("Content-Length")[0])

                print "Downloading {0} ({1} bytes)".format(url, file_size)
                bytes_downloaded = 0
                block_size = 8192

                while True:
                    buff = u.read(block_size)  # buffer
                    if not buff:

                        raw_files.append(local_file_name)

                        print "Succeeded at downloading {}".format(file_name)
                        break

                    bytes_downloaded += len(buff)
                    f.write(buff)
                    bytes_remaining = float(bytes_downloaded) / file_size
                    if config.DOWNLOAD_PROGRESS:
                        # TODO: Improve this progress indicator by making
                        # it more readable and user-friendly.
                        status = r"{0} [{1:.2%}]".format(bytes_downloaded,
                                                         bytes_remaining)
                        status = status + chr(8) * (len(status) + 1)
                        sys.stdout.write(status)

            return raw_files

        except OSError:
            print "Problem writing to {}.".format(config.LOCAL_DOWNLOAD_DIR)


def expand_qcow(image, size="+10G"):
    """Expand the storage for a qcow image. Currently only used for Atomic
    Hosts."""

    subprocess.call(['qemu-img',
                     'resize',
                     image,
                     size])

    print "Resized image for Atomic testing..."
    return


def create_user_data(path, password, overwrite=False, atomic=False):
    """Save the right  password to the 'user-data' file needed to
    emulate cloud-init. Default username on cloud images is "fedora"

    Will not overwrite an existing user-data file unless
    the overwrite kwarg is set to True."""

    if atomic:
        file_data = config.ATOMIC_USER_DATA % password

    else:
        file_data = config.USER_DATA % password

    if os.path.isfile(path + '/meta/user-data'):
        if overwrite:

            with open(path + '/meta/user-data', 'w') as user_file:
                user_file.write(file_data)

                return "user-data file generated."
        else:
            return "user-data file already exists"

    with open(path + '/meta/user-data', 'w') as user_file:
        user_file.write(file_data)

    return "user-data file generated."


def create_meta_data(path, hostname, overwrite=False):
    """Save the required hostname data to the 'meta-data' file needed to
    emulate cloud-init.

    Will not overwrite an existing user-data file unless
    the overwrite kwarg is set to True."""

    file_data = config.META_DATA % hostname

    if os.path.isfile(path + '/meta/meta-data'):
        if overwrite:

            with open(path + '/meta/meta-data', 'w') as meta_data_file:
                meta_data_file.write(file_data)

                return "meta-data file generated."
        else:
            return "meta-data file already exists"

    with open(path + '/meta/meta-data', 'w') as meta_data_file:
        meta_data_file.write(file_data)

    return "meta-data file generated."


def create_seed_img(meta_path, img_path):
    """Create a virtual filesystem needed for boot with virt-make-fs on a given
    path (it should probably be somewhere in '/tmp'."""

    make_image = subprocess.call(['virt-make-fs',
                                  '--type=msdos',
                                  '--label=cidata',
                                  meta_path,
                                  img_path + '/seed.img'])

    if make_image == 0:
        return "seed.img created at %s" % img_path

    return "creation of the seed.img failed."


def download_initrd_and_kernel(qcow2_image, path):
    """Download the necessary kernal and initrd for booting a specified cloud
    image. Returns a dict {'kernel': '', 'initrd': ''} after the download
    is completed."""

    subprocess.call(['virt-builder', '--get-kernel', qcow2_image], cwd=path)

    result = {}

    try:
        result['kernel'] = glob.glob("%s/*vmlinuz*" % path)[0]
        result['initrd'] = glob.glob("%s/*initramfs*" % path)[0]

    except IndexError:
        print "Unable to find kernel or initrd, did they download?"
        return

    return result


def boot_image(
        qcow2, seed, initrd=None, kernel=None, ram=1024, graphics=False,
        vnc=False, atomic=False, port=''):
    """Boot the cloud image redirecting local port 8888 to 80 on the vm as
    well as local port 2222 to 22 on the vm so http and ssh can be accessed."""

    boot_args = ['/usr/bin/qemu-kvm',
                 '-m',
                 str(ram),
                 '-drive',
                 'file=%s,if=virtio' % qcow2,
                 '-drive',
                 'file=%s,if=virtio' % seed,
                 '-redir',
                 'tcp:%s::22' % port,
                 ]

    if not atomic:
        boot_args.extend(['-kernel',
                          '%s' % kernel,
                          '-initrd',
                          '%s' % initrd,
                          '-append',
                          'root=/dev/vda1 ro ds=nocloud-net'
                          ])

    if graphics:
        boot_args.extend(['-nographic'])

    if vnc:
        boot_args.extend(['-vnc', '0.0.0.0:1'])

    print ' '.join(boot_args)
    vm = subprocess.Popen(boot_args)

    print "Successfully booted your local cloud image!"
    print "PID: %d" % vm.pid

    return vm


def build_and_run(
        image_url, ram=1024, graphics=False, vnc=False, atomic=False, port='', temppath='/tmp'):
    """Run through all the steps."""

    print "cleaning and creating dirs..."
    clean_dirs()
    create_dirs()

    base_path = temppath

    # Create cloud-init data
    print "Creating meta-data..."
    create_user_data(base_path, "passw0rd", atomic=atomic)
    create_meta_data(base_path, "testCloud")

    create_seed_img(base_path + '/meta', base_path)

    # Download image and get kernel/initrd

    image_file = os.path.join(base_path, image_url.split('/')[-1])

    if not os.path.isfile(image_file):
        print "downloading new image..."
        image = koji_download([image_url], temppath)[0]

        if atomic:
                expand_qcow(image)
    else:
        print "using existing image..."
        image = image_file

    if not atomic:
        external = download_initrd_and_kernel(image, base_path)

    if atomic:
        vm = boot_image(image,
                        base_path + '/seed.img',
                        ram=ram,
                        graphics=graphics,
                        vnc=vnc,
                        atomic=atomic,
                        port=port)
    else:
        vm = boot_image(image,
                        base_path + '/seed.img',
                        external['initrd'],
                        external['kernel'],
                        ram=ram,
                        graphics=graphics,
                        vnc=vnc,
                        port=port)

    return vm


def create_dirs():
    """Create the dirs in /tmp that we need to store things."""
    os.makedirs('/tmp/testCloud/meta')
    return "Created tmp directories."


def clean_dirs():
    """Remove dirs after a test run."""
    if os.path.exists('/tmp/testCloud'):
        shutil.rmtree('/tmp/testCloud')
    return "All cleaned up!"


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("url",
                        help="URL to qcow2 image is required.",
                        type=str)
    parser.add_argument("--ram",
                        help="Specify the amount of ram for the VM.",
                        type=int,
                        default=512)
    parser.add_argument("--no-graphic",
                        help="Turn off graphical display.",
                        action="store_true")
    parser.add_argument("--vnc",
                        help="Turns on vnc at :1 to the instance.",
                        action="store_true")
    parser.add_argument("--atomic",
                        help="Use this flag if you're booting an Atomic Host.",
                        action="store_true")

    args = parser.parse_args()

    gfx = False
    vnc = False
    atomic = False

    if args.no_graphic:
        gfx = True

    if args.vnc:
        vnc = True

    if args.atomic:
        atomic = True

    build_and_run(args.url, args.ram, graphics=gfx, vnc=vnc, atomic=atomic)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright © 2015  Kushal Das <kushaldas@gmail.com>
# Copyright © 2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2, or (at your option) any later
# version.  This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.  You
# should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

'''
Ukhra database model.
'''

__requires__ = ['SQLAlchemy >= 0.7', 'jinja2 >= 2.4']
import pkg_resources

import datetime
import logging
import time


import sqlalchemy as sa
from sqlalchemy import create_engine
from sqlalchemy.ext.associationproxy import association_proxy
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm import relation
from sqlalchemy.orm import backref
from sqlalchemy.orm.collections import attribute_mapped_collection
from sqlalchemy.orm.collections import mapped_collection
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql import and_
from sqlalchemy.sql.expression import Executable, ClauseElement

BASE = declarative_base()

ERROR_LOG = logging.getLogger('ukhra.lib.model')

# # Apparently some of our methods have too few public methods
# pylint: disable=R0903
# # Others have too many attributes
# pylint: disable=R0902
# # Others have too many arguments
# pylint: disable=R0913
# # We use id for the identifier in our db but that's too short
# pylint: disable=C0103
# # Some of the object we use here have inherited methods which apparently
# # pylint does not detect.
# pylint: disable=E1101


def create_tables(db_url, alembic_ini=None, debug=False):
    """ Create the tables in the database using the information from the
    url obtained.

    :arg db_url, URL used to connect to the database. The URL contains
        information with regards to the database engine, the host to
        connect to, the user and password and the database name.
          ie: <engine>://<user>:<password>@<host>/<dbname>
    :kwarg alembic_ini, path to the alembic ini file. This is necessary
        to be able to use alembic correctly, but not for the unit-tests.
    :kwarg debug, a boolean specifying wether we should have the verbose
        output of sqlalchemy or not.
    :return a session that can be used to query the database.

    """
    engine = create_engine(db_url, echo=debug)
    BASE.metadata.create_all(engine)
    if db_url.startswith('sqlite:'):
        # Ignore the warning about con_record
        # pylint: disable=W0613
        def _fk_pragma_on_connect(dbapi_con, con_record):
            ''' Tries to enforce referential constraints on sqlite. '''
            dbapi_con.execute('pragma foreign_keys=ON')
        sa.event.listen(engine, 'connect', _fk_pragma_on_connect)

    if alembic_ini is not None:  # pragma: no cover
        # then, load the Alembic configuration and generate the
        # version table, "stamping" it with the most recent rev:

        # Ignore the warning missing alembic
        # pylint: disable=F0401
        from alembic.config import Config
        from alembic import command
        alembic_cfg = Config(alembic_ini)
        command.stamp(alembic_cfg, "head")

    scopedsession = scoped_session(sessionmaker(bind=engine))
    return scopedsession


def drop_tables(db_url, engine):  # pragma: no cover
    """ Drops the tables in the database using the information from the
    url obtained.

    :arg db_url, URL used to connect to the database. The URL contains
    information with regards to the database engine, the host to connect
    to, the user and password and the database name.
      ie: <engine>://<user>:<password>@<host>/<dbname>
    """
    engine = create_engine(db_url)
    BASE.metadata.drop_all(engine)


class Job(BASE):
    "Each job run in the system"
    __tablename__ = 'job'

    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.String(255), nullable=False)
    image = sa.Column(sa.String(255), nullable=False)
    ram = sa.Column(sa.String(255), nullable=False)
    user = sa.Column(sa.String(255), nullable=False)
    password = sa.Column(sa.String(255), nullable=False)
    status = sa.Column(sa.Boolean, default=False)
    when = sa.Column(sa.DateTime, default=datetime.datetime.now())

    def __repr__(self):
        return '<Job(id=%d, name=%s, image=%s, ram=%s, user=%s, password=%s)>' % \
            (self.id, self.name, self.image, self.ram, self.user, self.password)


class Result(BASE):
    "Comments on bugs."
    __tablename__ = 'result'

    id = sa.Column(sa.Integer, primary_key=True)
    job_id = sa.Column(
        sa.Integer, sa.ForeignKey('job.id'), nullable=False)
    command = sa.Column(sa.String(255), nullable=False)
    output = sa.Column(sa.TEXT)
    return_code = sa.Column(sa.String(5), nullable=False)
    status = sa.Column(sa.Boolean)

    # Relations
    job = relation(
        'Job',
        foreign_keys=[job_id], remote_side=[Job.id],
        backref=backref('results')
    )

    def __repr__(self):
        return u'<Result(%d, %s, %s)>' % (self.id, self.command, self.status)


#!/usr/bin/env python
"""Factorial project"""
from setuptools import find_packages, setup

setup(name = 'tunir',
    version = '0.7',
    description = "Simple CI system.",
    long_description = "A simple CI system which can be maintained.",
    platforms = ["Linux"],
    author="Kushal Das",
    author_email="kushaldas@gmail.com",
    url="http://tunir.rtfd.org",
    license = "GPLv2+",
    packages=find_packages(),
    data_files=[('share/tunir',
                  ['createports.py', 'default.json',
                    'default.txt', 'vgt.json', 'vgt.txt']),
        ('share/man/man8/', ['tunir.8'])],
    entry_points = {
          'console_scripts': [
              'tunir = tunirlib:startpoint'
          ]
      }
    )
